import json
import os
import random
import sys
import time
import uuid

import requests

sys.path.append(os.getcwd())
from common.video_item import VideoItem
from common import PiaoQuanPipeline, AliyunLogger, tunnel_proxies, get_redirect_url
from common.mq import MQ


class Lymrzhspcheduling(object):
    def __init__(self, platform, mode, rule_dict, user_list, env):
        self.platform = platform
        self.mode = mode
        self.rule_dict = rule_dict
        self.user_list = user_list
        self.env = env
        self.download_cnt = 0
        self.mq = MQ(topic_name="topic_crawler_etl_" + self.env)
        self.limit_flag = False

    def get_videoList(self):

        headers = {
            'Host': 'mini.vvuiiu.cn',
            'accept': '*/*',
            'content-type': 'application/json',
            'accept-language': 'zh-cn',
            'user-agent': 'Mozilla/5.0 (iPhone; CPU iPhone OS 11_3 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E217 MicroMessenger/6.8.0(0x16080000) NetType/WIFI Language/en Branch/Br_trunk MiniProgramEnv/Mac',
            'referer': 'https://servicewechat.com/wx31e822f225bbb1a8/10/page-frame.html'
        }

        page = 1
        while True:
            time.sleep(random.randint(1, 10))
            try:
                if self.limit_flag:
                    message = "本轮已经抓取到足够的数据，自动退出\t{}".format(self.download_cnt)
                    AliyunLogger.logging(
                        code="2000",
                        platform=self.platform,
                        env=self.env,
                        mode=self.mode,
                        message=message
                    )
                    return
                else:
                    url = "https://mini.vvuiiu.cn/article/getArticleList"
                    payload = json.dumps({
                        "page": page,
                        "size": 5,
                        "category_id": "393817",
                        "from_type": 1,
                        "uuid": "70ea355e-20a7-43ed-ad11-e749fd2b147a",
                        "openid": "oyPMm4xW9jr-nkLQqKh3KXh2ZJq4",
                        "platform": 35,
                        "appid": "wx31e822f225bbb1a8"
                    })
                    response = requests.request("POST", url, headers=headers, data=payload)
                    video_list = response.json()['data']['list']
                    page += 1
                    if video_list:
                        for index, video_obj in enumerate(video_list, 1):
                            try:
                                AliyunLogger.logging(
                                    code="1001",
                                    platform=self.platform,
                                    mode=self.mode,
                                    env=self.env,
                                    message="扫描到一条视频",
                                    data=video_obj
                                )
                                self.process_video_obj(video_obj)
                            except Exception as e:
                                AliyunLogger.logging(
                                    code="3000",
                                    platform=self.platform,
                                    mode=self.mode,
                                    env=self.env,
                                    message="抓取第{}条的时候出现问题， 报错信息是{}".format(index, e)
                                )
                    else:
                        AliyunLogger.logging(
                            code="2000",
                            platform=self.platform,
                            mode=self.mode,
                            env=self.env,
                            message="视频已经抓完",
                        )
                        return
            except Exception as e:
                AliyunLogger.logging(
                    code="3000",
                    platform=self.platform,
                    mode=self.mode,
                    env=self.env,
                    message="抓取推荐页的时候出现错误， 报错信息是{}".format(e)
                )

    def process_video_obj(self, video_obj):
        trace_id = self.platform + str(uuid.uuid1())
        video_id = video_obj['id']
        video_url = video_obj['video_url']
        video_url = get_redirect_url(video_url)
        our_user = random.choice(self.user_list)
        item = VideoItem()
        item.add_video_info("user_id", our_user["uid"])
        item.add_video_info("user_name", our_user["nick_name"])
        item.add_video_info("video_id", video_id)
        item.add_video_info("video_title", video_obj["title"])
        item.add_video_info("video_url", video_url)
        item.add_video_info("cover_url", video_obj["image_path"])
        item.add_video_info("avatar_url", video_obj["image_path"])
        item.add_video_info("out_video_id", video_id)
        item.add_video_info("platform", self.platform)
        item.add_video_info("strategy", self.mode)
        item.add_video_info("session", "{}-{}".format(self.platform, int(time.time())))
        mq_obj = item.produce_item()
        pipeline = PiaoQuanPipeline(
            platform=self.platform,
            mode=self.mode,
            rule_dict=self.rule_dict,
            env=self.env,
            item=mq_obj,
            trace_id=trace_id,
        )
        if pipeline.process_item():
            self.download_cnt += 1
            self.mq.send_msg(mq_obj)
            AliyunLogger.logging(
                code="1002",
                platform=self.platform,
                mode=self.mode,
                env=self.env,
                message="成功发送至 ETL",
                data=mq_obj,
                trace_id=trace_id
            )
            if self.download_cnt >= int(
                self.rule_dict.get("videos_cnt", {}).get("min", 200)
            ):
                self.limit_flag = True

if __name__ == "__main__":
    ZL = Lymrzhspcheduling(
        platform="Lymrzf",
        mode="recommend",
        env="dev",
        rule_dict={},
        user_list=[{'nick_name': "Ivring", 'uid': "1997"}, {'nick_name': "paul", 'uid': "1998"}]
    )
    ZL.get_videoList()